package com.qmth.themis.backend.api;

import com.qmth.themis.business.annotation.ApiJsonObject;
import com.qmth.themis.business.annotation.ApiJsonProperty;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.enums.FinishTypeEnum;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.ServletUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.enums.MqTagEnum;
import com.qmth.themis.mq.enums.MqTopicEnum;
import com.qmth.themis.mq.service.MqDtoService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 监考信息 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "监考信息Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/invigilate")
public class TIeInvigilateController {
    private final static Logger log = LoggerFactory.getLogger(TIeInvigilateController.class);

    @Resource
    MqDtoService mqDtoService;

    @ApiOperation(value = "实时监控台列表接口")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list() {
        log.info("invigilate list is come in");
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "实时监控台视频列表接口")
    @RequestMapping(value = "/list/video", method = RequestMethod.POST)
    public Result listVideo() {
        log.info("invigilate listVideo is come in");
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "强制/手动交卷接口")
    @RequestMapping(value = "/finish", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result finish(@ApiJsonObject(name = "invigilateFinish", value = {
            @ApiJsonProperty(key = "examRecordId", description = "考试记录id"),
            @ApiJsonProperty(key = "type", description = "收卷类型，手动/强制")
    }) @ApiParam(value = "考试记录信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("recordId")) || Objects.equals(mapParameter.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        List<Long> recordIdList = (List<Long>) mapParameter.get("recordId");
        if (Objects.isNull(mapParameter.get("type")) || Objects.equals(mapParameter.get("type"), "")) {
            throw new BusinessException(ExceptionResultEnum.FINISH_TYPE_IS_NULL);
        }
        FinishTypeEnum type = FinishTypeEnum.valueOf(String.valueOf(mapParameter.get("type")));
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        //发送mq给客户端强制收卷
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.oe.name(), JacksonUtil.parseJson(recordIdList), type.ordinal() == FinishTypeEnum.INTERRUPT.ordinal() ? MqEnum.WEBSOCKET_MONITOR_FINISH_LOG : MqEnum.WEBSOCKET_HAND_FINISH_LOG, String.valueOf(tbUser.getId()), mapParameter, tbUser.getName());
        mqDtoService.assembleSendOneWayMsg(mqDto);
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "监考消息通知接口")
    @RequestMapping(value = "/notice", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result notice(@ApiJsonObject(name = "invigilateNotice", value = {
            @ApiJsonProperty(key = "recordId", description = "考试记录id"),
            @ApiJsonProperty(key = "type", description = "消息类型，text/audio")
    }) @ApiParam(value = "考试记录信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("recordId")) || Objects.equals(mapParameter.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = Long.parseLong(String.valueOf(mapParameter.get("recordId")));
        if (Objects.isNull(mapParameter.get("type")) || Objects.equals(mapParameter.get("type"), "")) {
            throw new BusinessException(ExceptionResultEnum.FINISH_TYPE_IS_NULL);
        }
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }
}
